/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */

/// Touch the velocities field.

#version 110

// Use low precision.
// precision lowp float;

uniform sampler2D control;      /* Flow control plane.                  */
uniform sampler2D paper;        /* Paper texture.                       */
uniform sampler2D oldpatch;     /* Patch to be overwritten.             */

uniform vec2 brush_uv;          /* Brush velocity.                      */
uniform vec2 paper_offset;      /* Offset of a paper texture.           */
uniform vec2 paper_scale;       /* Scaling factor for a paper texture.  */


void main (void) {
  //vec4 patch = texture2D (oldpatch, gl_TexCoord[1].st);

    float ctl_src = texture2D (control, gl_TexCoord[0].st).x;
  //float height  = texture2D (paper, fract (gl_TexCoord[0].st * paper_scale + paper_offset)).x;

    // Source control primitive as mask.
    if (abs (ctl_src) < 0.01)
        discard;

    gl_FragColor = clamp (
        vec4 (
            -brush_uv.y,    // dot (brush_uv, vec2 (+0.0, -1.0)),
             brush_uv.x,    // dot (brush_uv, vec2 (+1.0, +0.0)),
             brush_uv.y,    // dot (brush_uv, vec2 (+0.0, +1.0)),
            -brush_uv.x     // dot (brush_uv, vec2 (-1.0, +0.0))
        ),
        0.0, 1.0
    );
}
