/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */

// Render the velocities field.

#version 110

uniform sampler2D Meta0;       /* Patch metadata.  */
uniform float grid;            /* Grid step.       */
uniform vec4 color;            /* Vectors color.   */

void main (void) {
    vec2 tx = 2.0 * gl_TexCoord[0].st;

    /* Get the grid point */
    vec2 agtx = (floor (tx / grid) + 0.5) * grid;
    vec2 gtx  = agtx;

    /* Get the vector at a grid point */
    /* (left-top sub-patch)           */
    vec4 vee = texture2D (Meta0, 0.5 * agtx);
    vec2 uv  = vec2 (vee.g - vee.a, vee.b - vee.r);

    //uv = normalize (vec2 (-.8, -1.0));

    /* Calculate a distance from p0 to line p1->p2 */
    vec2 p0 = tx;
    vec2 p1 = gtx;
    vec2 p2 = gtx + uv * grid * 4.0;

    vec2 r  = p1 - p0;
    vec2 v  = vec2 (p2.y - p1.y, p1.x - p2.x);

    /* Paint if the fragment is on vector */
    if (length (v) > 0.001 &&
        distance (gtx, p0) < length (p2 - p1)) {
        float d = abs (dot (normalize (v), r));
        float a = dot (p2 - p1, r);

        if (d > 0.0016 || a > 0.0)
            discard;

        gl_FragColor = color;
    } else
        discard;
}
