/***
 *  Copyright (C) 2009 Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */

// Mass advection iteration
// on an LBE velocity-grid.

#version 110


uniform sampler2D src;      /** Source texture.     */
uniform sampler2D meta0;    /** Meta channel.       */
uniform vec2 texstep_x;
uniform vec2 texstep_y;
uniform vec2 texoffset;
uniform float dt;

const vec4 g_ONE  = vec4 (+1.0, +1.0, +1.0, +1.0);
const vec4 g_0101 = vec4 (+0.0, +1.0, +0.0, -1.0);
const vec4 g_1010 = vec4 (-1.0, +0.0, +1.0, +0.0);

void main (void) {
    vec2 tx0 = gl_TexCoord[0].st * 0.5 + texoffset;
    vec2 tx1 = gl_TexCoord[0].st * 0.5;
    vec4 e  = texture2D (meta0, tx1);
    vec4 v0 = texture2D (src,   tx0);

    // Center velocity.
    vec2 u = vec2 (e.g - e.a, e.b - e.r);
/*
    vec2 u = e.r * vec2 (+0.0, -1.0) +
             e.g * vec2 (+1.0, +0.0) +
             e.b * vec2 (+0.0, +1.0) +
             e.a * vec2 (-1.0, +0.0);
*/

    vec4 v1 = texture2D (src, tx0 - 0.02 * u);

    gl_FragData[0] = mix (v0, v1, 0.5);
}
