/***
 *  Copyright (C) 2009 Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */

// Velocities self-advection iteration
// on an LBE grid.

#version 110


uniform sampler2D meta0;
uniform vec2 texstep_x;
uniform vec2 texstep_y;
uniform float dt;

const vec4 g_ONE  = vec4 (1.0, 1.0, 1.0, 1.0);
const vec4 g_0101 = vec4 (0.0, 1.0, 0.0, 1.0);
const vec4 g_1010 = vec4 (1.0, 0.0, 1.0, 0.0);

const float relaxation = 0.1;


void main (void) {
    // Neighbouring cells.
    vec2 tx0 = gl_TexCoord[0].st * 0.5 + vec2 (0.5, 0.0);
    vec4 e = texture2D (meta0, tx0);
    vec4 l = texture2D (meta0, tx0 - texstep_x * dt);
    vec4 r = texture2D (meta0, tx0 + texstep_x * dt);
    vec4 t = texture2D (meta0, tx0 - texstep_y * dt);
    vec4 b = texture2D (meta0, tx0 + texstep_y * dt);

    // Center velocity.
    vec2 u = e.r * vec2 (+0.0, -1.0) +
             e.g * vec2 (+1.0, +0.0) +
             e.b * vec2 (+0.0, +1.0) +
             e.a * vec2 (-1.0, +0.0);
 // u = vec2 (dot (e, g_0101). dot (e, g_1010));

    vec4 d = vec4 (
        dot (vec2 (+0.0, -1.0), u),
        dot (vec2 (+1.0, +0.0), u),
        dot (vec2 (+0.0, +1.0), u),
        dot (vec2 (-1.0, +0.0), u)
    );

    // Equilibrium forces.
    vec4 eq = 0.2 * (g_ONE + 3.0 * d +
                             4.5 * d * d -
                             1.5 * dot (u, u)
                    );

    // Stream and collide.
    gl_FragData[0] = clamp (
        e + (1.0 - relaxation) * (vec4 (b.r, l.g, t.b, r.a) + relaxation * eq) * dt,
            //(b.r - e.r) * dt,
            //(l.g - e.g) * dt,
            //(t.b - e.b) * dt,
            //(r.a - e.a) * dt
        //),
        0.0, 1.0
    );
}
