/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */

// Paints drying iteration.

#version 140

uniform sampler2D wet;
uniform sampler2D dry;
uniform sampler2D ctl;
uniform sampler2D paper;
uniform vec2      paper_off;
uniform vec2      paper_scale;
uniform float     dt;
uniform float     granulation;

centroid nonperspective in vec2 g_texcoord0;
centroid nonperspective in vec2 g_texcoord1;

out vec4 result0;
out vec4 result1;
out vec4 result2;


void main (void) {
    vec4 d = texture2D (dry,    g_texcoord0);
    vec4 s = texture2D (wet,    g_texcoord0);
    vec4 c = texture2D (ctl,    g_texcoord0);
    vec4 h = texture2D (paper,  fract (g_texcoord1 * paper_scale + paper_off));

    float rate = 1.0 - granulation * (1.0 - h.x);
    vec4  flow = s * 0.05 * rate * dt;

    result0 = clamp (s - flow, 0.0, 1.0);
    result1 = clamp (d + flow, 0.0, 1.0);
    result2 = vec4 (
        c.r,
        c.g,
        max (c.b - 0.05 * dt, 0.0),
        1.0
    );
}
